﻿-- Author      : RisM
-- Create Date : 6/28/2009 4:03:23 PM

SpeakinSpell = LibStub("AceAddon-3.0"):GetAddon("SpeakinSpell")
local L			= LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell", false)
local HELPFILE	= LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell_HELPFILE",			false)
local DEFAULT_SPEECHES	= LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell_DEFAULT_SPEECHES",	false)

-------------------------------------------------------------------------------
-- SLASH COMMAND HANDLERS
-------------------------------------------------------------------------------


function SpeakinSpell:RegisterChatCommands()
	-- NOTE: support both localized and default English slash commands
	local SlashCommands = {
		L["/SpeakinSpell"],
		  "/SpeakinSpell",
		L["/speakinspell"],
		  "/speakinspell",
		L["/SS"],
		  "/SS",
		L["/ss"],
		  "/ss",
	}
	
	for k,v in pairs(SlashCommands) do
		self:RegisterChatCommand(v,"OnSlashCommand")
	end
end


function SpeakinSpell:OnSlashCommand(input)
	local funcname = "OnSlashCommand"

	input = string.lower(input)
	self:DebugMsg(funcname,"OnSlashCommand:"..input)
	
	-- if nothing is specified, then choose a default command
	if input == nil or input == "" then
		input = "options"
	else
		-- check for commands that require special parsing
		
		-- check for /ss ad /w playername
		local whisperad = self:StartsWith(input, "ad /w")
		local len = string.len("ad /w")
		if not whisperad then -- Localized check
			whisperad = self:StartsWith(input, L["/ad /w"])
			len = string.len(L["/ad /w"])
		end
		if whisperad then
			local target = string.sub(input, len+2, -1) --from len to the end of the string
			self:Advertise("WHISPER", target)
			return
		end
		
		-- check for /ss macro something
		local macro = self:StartsWith(input, "macro")
		if not macro then -- Localized check
			macro = self:StartsWith(input, L["/macro"])
		end
		if macro then
			self:DebugMsg(funcname, "User Macro input:"..input)
			local DetectedEventStub = {
				name = input, -- use the entire lowercase input = "macro something"
				rank = "",
				type = "MACRO",
				caster = UnitName("player"),
				target = self:GetDefaultTarget(),
			}
			self:OnEventDetected( DetectedEventStub )
			return
		end
		
	end
	
	-- table of supporting slash commands
	-- NOTE: support both English and current locale
	--	non-English game clients can typically use both translated and untranslated slash commands
	--	for built-in functions like /bg chat, etc, so it makes sense here as well
	local SlashCommandTable = {
		-- /ss options	- also show options
		[L["/options"]] = function() self:ShowOptions() end,
		[   "options" ] = function() self:ShowOptions() end,
		
		-- /ss create	- also Create New... interface
		[L["/create"]] = function() self:ShowCreateNew() end,
		[   "create" ] = function() self:ShowCreateNew() end,
		
		-- /ss create	- also Create New... interface
		[L["/messages"]] = function() self:ShowMessageOptions() end,
		[   "messages" ] = function() self:ShowMessageOptions() end,
		
		-- /ss help 	- show the help
		[L["/help"]] = function() self:ShowHelp() end,
		[   "help" ] = function() self:ShowHelp() end,
		
		-- /ss reset	- reset to defaults
		[L["/reset"]] = function() self:ResetToDefaults(nil) end,
		[   "reset" ] = function() self:ResetToDefaults(nil) end,
		
		-- /ss advertise- show an advertisement for SpeakinSpell
		[L["/advertise"]] = function() self:Advertise("") end,
		[   "advertise" ] = function() self:Advertise("") end,
		
		[L["/ad /s"]] = function() self:Advertise("SAY") end,
		[   "ad /s" ] = function() self:Advertise("SAY") end,
		
		[L["/ad /p"]] = function() self:Advertise("PARTY") end,
		[   "ad /p" ] = function() self:Advertise("PARTY") end,
		
		[L["/ad /ra"]] = function() self:Advertise("RAID") end,
		[   "ad /ra" ] = function() self:Advertise("RAID") end,
	}
	
	-- what did the user enter?
	Func = SlashCommandTable[input]
	
	-- process the command
	if Func then
		Func(input)
	else -- in all other cases, show help
		-- go straight to the slash commands page
		-- NOTE: A Blizzard Bug won't open a tier 3 page unless the tree has already been expanded to show tier 2
		--		so we'll try to show the parent frame first
		-- NOTE: I tried to work aroubd it by first opening the root page, but it stayed there
		--		I guess there's a timing issue with the windows messages (?) that makes it stick on the first one
		--		or maybe you're not allowed and it stops executing when you call this function (?)
		--self:ShowHelp() -- whatever, it didn't work, see notes above
		-- the slash commands page is now tier 2 to avoid this issue and other general bugginess with tier 3 pages
		InterfaceOptionsFrame_OpenToCategory(HELPFILE.SLASH_COMMANDS)
	end
end



function SpeakinSpell:Advertise(Channel, msgtarget)
	-- advertise into a channel for the kind of group you are in
	-- unless a channel has been specified
	if Channel == "" or Channel == nil then
		if (IsActiveBattlefieldArena()) then
			Channel = "PARTY" -- use Party chat for advertisements in Arena
		elseif (GetNumBattlefieldScores() > 0) then
			Channel = "SAY" -- don't advertise in BG chat, that would be annoying
		elseif (GetNumRaidMembers() > 0) then
			Channel = "RAID"
		elseif (GetNumPartyMembers() > 0) then
			Channel = "PARTY"
		else -- solo
			Channel = "SAY"
		end
	end
	
	-- if target is literally <target> or unspecified, then get the current target
	-- /ss advertise /w <target>
	-- /ss advertise /w
	if not msgtarget or msgtarget == "" or msgtarget == "<target>" or msgtarget == "<"..L["<target>"]..">" then
		msgtarget = self:GetDefaultTarget()
	end
	
	-- select a random ad that was not used last time
	local sizelist = #(DEFAULT_SPEECHES.ADVERTISEMENTS)
	local tempnum = math.random(1, sizelist);
	
	msg = DEFAULT_SPEECHES.ADVERTISEMENTS[tempnum]
	assert(msg)
	assert(type(msg) == "string")
	
	-- avoid repeating the same message we used last time
	-- unless there's only one in the list
	while sizelist > 1 and msg == self.RuntimeData.LastAd do
		tempnum = math.random(1, sizelist);
		msg = DEFAULT_SPEECHES.ADVERTISEMENTS[tempnum]
	end
	
	-- remember we selected this, so we don't repeat it next time
	self.RuntimeData.LastAd = msg

	-- process substitution variables on the randomized advertisements
	--TODOFUTURE: ?? change ads to a customizable macro event, to share more code with OnEventDetected
	local DetectedEventStub = {
		name = "ad",
		rank = "Silly",
		target = msgtarget,
		caster = UnitName("player"),
		type = "MACRO"
	}
	local DetectedEvent = self:CreateDetectedEvent( DetectedEventStub )
	msg = self:ProcessSubstitutionVars(msg, DetectedEvent)
	
	-- speak in chat
	SendChatMessage(msg, Channel, nil, msgtarget)
	SendChatMessage(SpeakinSpell.URL, Channel, nil, msgtarget)	
end
