﻿-- Author      : RisM
-- Create Date : 6/28/2009 3:54:18 PM

SpeakinSpell = LibStub("AceAddon-3.0"):GetAddon("SpeakinSpell")
local L	= LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell", false)

-------------------------------------------------------------------------------
-- UTILITY FUNCTIONS
-------------------------------------------------------------------------------


-- Import new contents from Source into existing table Dest
function SpeakinSpell:AddTable(Dest, Source)
	-- create the destination if it doesn't exist
	if Dest == nil then
		Dest = {}
	end
	assert( type(Dest) == "table" )
	
	-- if there's no source, then there's nothing to add
	if Source == nil then
		-- nothing to add
		return Dest
	end
	
	-- add the contents of Source, one item at a time
	for k,v in pairs(Source) do
		-- dig into embedded tables to add those too, not just create references
		if type(v) == "table" then
			-- NOTE: safety checks above will ensure we create Dest[k] if necessary
			Dest[k] = self:AddTable(Dest[k], v)
		else
			Dest[k] = v
		end
	end
	
	return Dest
end


-- create a deep copy of the table and return it
function SpeakinSpell:CopyTable(SourceTable)
	-- adding SourceTable to a new empty table will create a copy
	return self:AddTable(nil, SourceTable)
end


function SpeakinSpell:GetDefaultTarget()
	-- try the currently selected target
	local target = UnitName("target")
	if target then
		self:DebugMsg(nil,"<target> is unknown, using selected:"..target)
		return target
	end
	
	-- nobody is selected, try focus
	target = UnitName("focus")
	if target then
		self:DebugMsg(nil,"<target> is unknown, using focus:"..target)
		return target
	end
	
	-- assume self-cast ability
	target = UnitName("player")
	self:DebugMsg(nil,"<target> is unknown, using self:"..target)
	
	return target
end


function SpeakinSpell:ShowOptions()
	-- this is probably not necessary, but make sure this is up to date before opening the options panel
	self:CurrentMessagesGUI_Refresh()
	
	-- open interface options to show SpeakinSpell
	InterfaceOptionsFrame_OpenToCategory(L["SpeakinSpell"])
end


function SpeakinSpell:ShowHelp()
	-- this is probably not necessary, but make sure this is up to date before opening the options panel
	self:CurrentMessagesGUI_Refresh()
	
	-- open interface options to show SpeakinSpell
	InterfaceOptionsFrame_OpenToCategory(L["SpeakinSpell Help"])
end


function SpeakinSpell:ShowCreateNew()
	-- this is probably not necessary, but make sure this is up to date before opening the options panel
	self:CurrentMessagesGUI_Refresh()
	
	-- open interface options to show SpeakinSpell
	InterfaceOptionsFrame_OpenToCategory(L["Create New..."])
end


function SpeakinSpell:ShowMessageOptions()
	-- this is probably not necessary, but make sure this is up to date before opening the options panel
	self:CurrentMessagesGUI_Refresh()
	
	-- open interface options to show SpeakinSpell
	InterfaceOptionsFrame_OpenToCategory(L["Message Settings TITLE"])
end


function SpeakinSpell:TitleCase(s)
	local first = string.sub(s,1,1)
	local rest = string.sub(s,2,-1)

	first = string.upper(first)
	
	rest = string.gsub(rest, " %a",
		function(match) 
			return string.upper(match)
		end
	)
	
	local result = first..rest
	return result
end


function SpeakinSpell:StartsWith(s, prefix)
	start = string.sub(s, 1, string.len(prefix))
	return (start == prefix)
end


function SpeakinSpell:EndsWith(s, suffix)
	strend = string.sub(s, -1 * string.len(suffix))
	return (strend == suffix)
end

